#include <avr/io.h>
#include <util/delay.h>
#include <inttypes.h>

//tryby pracy
#define MODE_FIRST 1
#define MODE_SECOND 2
#define MODE_THIRD 3
#define MODE_FOURTH 4
#define MODE_NONE 0

//deklaracje wyj sterujcych 
#define OUT1_1 PORTD|=(1<<PORTD6)
#define OUT2_1 PORTD|=(1<<PORTD7)
#define OUT3_1 PORTB|=(1<<PORTB0)
#define OUT4_1 PORTB|=(1<<PORTB1)

#define OUT1_0 PORTD&=~(1<<PORTD6)
#define OUT2_0 PORTD&=~(1<<PORTD7)
#define OUT3_0 PORTB&=~(1<<PORTB0)
#define OUT4_0 PORTB&=~(1<<PORTB1)

//wyjcia sterujce diod LED
#define LED_R PORTD|=(1<<PORTD0);PORTD&=~(1<<PORTD1)
#define LED_G PORTD|=(1<<PORTD1);PORTD&=~(1<<PORTD0)
#define LED_NONE PORTD&=~(1<<PORTD0);PORTD&=~(1<<PORTD1)


#define SWITCH bit_is_clear(PIND,PIND2) //wejcie impulsw
#define MODE_1 bit_is_clear(PINC,PINC3) //wejcia przecznika wyboru trybu pracy
#define MODE_2 bit_is_clear(PINC,PINC2)
#define MODE_3 bit_is_clear(PINC,PINC1)
#define MODE_4 bit_is_clear(PINC,PINC0)

#define TIME_SW 2000 //czas midzy kolejnymi naciniciami
#define DELAY_SW 250 //tyle milisekund czeka po wciniciu
#define SW_MAX 4 //tyle razy mona wcisn przycisk


//---------------------------------------------------------------
// Deklaracje zmiennych                                    
//---------------------------------------------------------------

volatile uint8_t mode = MODE_NONE; //wybrany tryb pracy
volatile uint8_t pulse = 0; //odczytana z przecznika liczba impulsw
volatile uint8_t sw_poprz = 1; //poprzedni stan przycisku, do wykrycia zmiany
volatile uint8_t next_off = 0; //jeeli 1 to nastpne wcinicie przycisku spowoduje wyczenie

volatile uint16_t time_cnt = 0; //licznik czasu midzy przyciniciami

//---------------------------------------------------------------
// GLWNA FUNKCJA PROGRAMU                                                   
//---------------------------------------------------------------
int main(void)
{

	DDRB = (1 << DDB0) | (1 << DDB1);
	PORTB = (1 << PORTB3) | (1 << PORTB4) | (1 << PORTB5);

	DDRC = 0;
	PORTC = (1 << PORTC0) | (1 << PORTC1) | (1 << PORTC2) | (1 << PORTC3);

	DDRD = (1 << DDD0) | (1 << DDD1) | (1 << DDD6) | (1 << DDD7);
	PORTD = (1 << PORTD2);


	while (1)
	{
		_delay_ms(1); //opnienie gwnej ptli
		
		//jeeli odliczanie czasu jest aktywne, odejmuj kolejne milisekundy
		if(time_cnt) 
		{
			time_cnt--; 
		}
		
		//uaktualnij stan zmiennej od przycisku, jeeli jest zwolniony
		if(!SWITCH)
		{
			sw_poprz = 1;
		}
	
		//odczytanie trybu pracy ze zworek
		//tylko jedna moe by naoona!
		if(MODE_1 && !MODE_2 && !MODE_3 && !MODE_4)
		{
			mode = MODE_FIRST;
		}
		else if(!MODE_1 && MODE_2 && !MODE_3 && !MODE_4)
		{
			mode = MODE_SECOND;
		}
		else if(!MODE_1 && !MODE_2 && MODE_3 && !MODE_4)
		{
			mode = MODE_THIRD;
		}
		else if(!MODE_1 && !MODE_2 && !MODE_3 && MODE_4)
		{
			mode = MODE_FOURTH;
		}
		else
		{
			mode = MODE_NONE;
		}
		
		//jeeli wykryto nieprawidowy tryb pracy, migaj diod i nic poza tym
		if(mode == MODE_NONE)
		{
			LED_R;
			_delay_ms(200);
			LED_G;
			_delay_ms(200);
			LED_NONE;
		}
		//w prawidowym trybie pracy
		else
		{
			//jeeli pierwszy lub nastpny raz wcinito przycisk
			if(SWITCH && sw_poprz == 1 && time_cnt < (TIME_SW-DELAY_SW) && pulse < SW_MAX && next_off == 0)
			{
				pulse++; //odczytaj ten impuls
				time_cnt = TIME_SW; //rozpocznij odliczanie
				LED_G; //zacz zielon diod
				sw_poprz = 0; //zapamitaj wcinicie przycisku
			}
			
			//jeeli wcinito przycisk zbyt wiele razy, wycz przekaniki
			if(SWITCH && sw_poprz == 1 && time_cnt < (TIME_SW-DELAY_SW) && pulse >= SW_MAX && next_off == 0)
			{
				LED_NONE; //wycz diod LED
				next_off = 0; pulse = 0; time_cnt = 0; //wyzeruj zmienne
				sw_poprz = 1; //zapamitaj wcinicie przycisku
				OUT1_0; OUT2_0; OUT3_0; OUT4_0; //wycz przekaniki
				time_cnt = TIME_SW; //rozpocznij odliczanie
			}
			
			
			//jeeli od ostatniego wcinicia mino ju duo czasu, nastpny impuls wyczy przekaniki
			if(pulse > 0 && time_cnt == 0 && next_off == 0)
			{
				LED_R; //dioda staje si czerwona
				next_off = 1; //uaktywnij moliwo wyczenia
			}
			
			//jeeli wcinito przycisk po bardzo dugim czasie a owietlenie jest zaczone
			if(SWITCH && sw_poprz == 1 && next_off == 1)
			{
				LED_NONE; //wycz diod LED
				next_off = 0; pulse = 0; time_cnt = 0; //wyzeruj zmienne
				sw_poprz = 1; //zapamitaj wcinicie przycisku
				OUT1_0; OUT2_0; OUT3_0; OUT4_0; //wycz przekaniki
				time_cnt = TIME_SW; //rozpocznij odliczanie
			}
			
			//w pierwszym trybie dziaania
			if(mode == MODE_FIRST)
			{
				if(pulse == 1){OUT1_1; OUT2_0; OUT3_0; OUT4_0;}
				if(pulse == 2){OUT1_0; OUT2_1; OUT3_0; OUT4_0;}
				if(pulse == 3){OUT1_0; OUT2_0; OUT3_1; OUT4_0;}
				if(pulse == 4){OUT1_0; OUT2_0; OUT3_0; OUT4_1;}
			}
			
			//w drugim trybie dziaania
			if(mode == MODE_SECOND)
			{
				if(pulse == 1){OUT1_1; OUT2_0; OUT3_0; OUT4_0;}
				if(pulse == 2){OUT1_1; OUT2_1; OUT3_0; OUT4_0;}
				if(pulse == 3){OUT1_1; OUT2_1; OUT3_1; OUT4_0;}
				if(pulse == 4){OUT1_1; OUT2_1; OUT3_1; OUT4_1;}
			}

			//w trzecim trybie dziaania
			if(mode == MODE_THIRD)
			{
				if(pulse == 1){OUT1_1; OUT2_1; OUT3_0; OUT4_0;}
				if(pulse == 2){OUT1_0; OUT2_1; OUT3_1; OUT4_0;}
				if(pulse == 3){OUT1_0; OUT2_0; OUT3_1; OUT4_1;}
				if(pulse == 4){OUT1_1; OUT2_0; OUT3_0; OUT4_1;}
			}
			
			//w czwartym trybie dziaania
			if(mode == MODE_FOURTH)
			{
				if(pulse == 1){OUT1_0; OUT2_1; OUT3_1; OUT4_0;}
				if(pulse == 2){OUT1_1; OUT2_0; OUT3_0; OUT4_1;}
				if(pulse == 3){OUT1_1; OUT2_1; OUT3_0; OUT4_0;}
				if(pulse == 4){OUT1_0; OUT2_0; OUT3_1; OUT4_1;}
			}	
		}
	}
return 0;
}





   
   
